import { dataProperty, SwiftUICommon, BaseUIDataDriver, NativeScriptViewHelper } from './common';
import { Utils } from '@nativescript/core';
export * from './common';
export * from './components';
let registeredSwiftUIViews;
export function registerSwiftUI(id, callback) {
    if (!registeredSwiftUIViews) {
        registeredSwiftUIViews = new Map();
    }
    registeredSwiftUIViews.set(id, callback);
}
export class UIDataDriver extends BaseUIDataDriver {
    constructor(swiftUIProvider, owner) {
        super(owner);
        this.swiftUIProvider = swiftUIProvider;
    }
    createNativeView() {
        const vc = this.swiftUIProvider;
        return vc.view;
    }
    registerEvents(callback) {
        this.swiftUIProvider.onEvent = (data) => {
            callback(data);
        };
    }
    updateData(data) {
        this.swiftUIProvider.updateDataWithData(data);
    }
    onEvent(data) {
        const owner = this.owner.get();
        if (owner) {
            owner.notify({
                eventName: SwiftUICommon.swiftUIEventEvent,
                data: Utils.dataDeserialize(data),
            });
        }
    }
}
export class SwiftUI extends SwiftUICommon {
    createNativeView() {
        const generator = registeredSwiftUIViews.get(this.swiftId);
        if (!generator) {
            console.warn('view not registered:', this.swiftId);
            return UIView.new();
        }
        this.driver = generator(this);
        return this.driver.createNativeView();
    }
    initNativeView() {
        if (!this.driver) {
            return;
        }
        this.driver.registerEvents((data) => {
            if (this.driver.onEvent) {
                this.driver.onEvent(data);
            }
            else {
                this.notify({
                    eventName: SwiftUI.swiftUIEventEvent,
                    data,
                });
            }
        });
    }
    disposeNativeView() {
        super.disposeNativeView();
        if (this.driver) {
            this.driver.destroyNativeView?.();
        }
    }
    [dataProperty.setNative](data) {
        this.updateData(data);
    }
    updateData(data) {
        this.driver?.updateData?.(data);
    }
}
// Possible NativeScriptView's by id
let registeredNativeScriptViews;
// NativeScriptView's by id which have been created and in use
let createdNativeScriptViews;
// Any custom lifecycle needed for views when paired with flavors
let customViewLifeCycle;
export class SwiftUIManager {
    /**
     * Register NativeScript Views by id for usage within SwiftUI
     * @param views map of id to NativeScript View
     */
    static registerNativeScriptViews(views, viewLifeCycle) {
        if (!registeredNativeScriptViews) {
            registeredNativeScriptViews = new Map();
            // ensure view lifecycle is setup with @nativescript/core factory
            NativeScriptViewFactory.shared.viewCreator = (id) => {
                // console.log("viewCreator:", id);
                // Each view is created with a unique instance id, with the registered id as the prefix for easy lookup
                const registerId = id.split('-')[0];
                if (registeredNativeScriptViews.has(registerId)) {
                    let view;
                    if (customViewLifeCycle) {
                        view = customViewLifeCycle.create(id, registeredNativeScriptViews.get(registerId));
                    }
                    else {
                        // TODO: xml view creation fallback
                    }
                    if (!createdNativeScriptViews) {
                        createdNativeScriptViews = new Map();
                    }
                    createdNativeScriptViews.set(id, view);
                    NativeScriptViewFactory.shared.views.setObjectForKey(NativeScriptViewHelper.generateNativeView(view, {}), id);
                }
            };
            NativeScriptViewFactory.shared.viewDestroyer = (id) => {
                // console.log("viewDestroyer:", id);
                const view = createdNativeScriptViews?.get(id);
                if (view) {
                    if (customViewLifeCycle) {
                        customViewLifeCycle.destroy(id);
                    }
                    NativeScriptViewHelper.disposeView(view);
                    createdNativeScriptViews.delete(id);
                    NativeScriptViewFactory.shared.views.removeObjectForKey(id);
                }
            };
        }
        if (viewLifeCycle) {
            customViewLifeCycle = viewLifeCycle;
        }
        const entries = Object.entries(views);
        entries.forEach((entry) => {
            registeredNativeScriptViews.set(entry[0], entry[1]);
        });
    }
}
export class WindowManager {
    static getWindow(id, isImmersive) {
        if (!WindowManager.currentWindows) {
            WindowManager.currentWindows = {};
        }
        if (!WindowManager.currentWindows[id]) {
            WindowManager.currentWindows[id] = new NativeScriptWindow(id, isImmersive);
        }
        return WindowManager.currentWindows[id];
    }
    static supportsMultipleScenes() {
        return UIApplication.sharedApplication.supportsMultipleScenes;
    }
}
export class NativeScriptWindow {
    constructor(id, isImmersive) {
        this.id = id;
        this.isImmersive = isImmersive;
    }
    open(props) {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowOpen', props);
        }
        return Promise.resolve();
    }
    close() {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowClose');
            NativeScriptWindowFactory.shared.removeWindowWithId(this.id);
            delete WindowManager.currentWindows[this.id];
        }
        return Promise.resolve();
    }
    update(props) {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowUpdate', props);
        }
        return Promise.resolve();
    }
    _updateWindowData(eventName, props) {
        if (props) {
            NativeScriptWindowFactory.shared.updateDataWithIdUpdates(this.id, props);
        }
        NSNotificationCenter.defaultCenter.postNotificationNameObjectUserInfo(eventName, null, Utils.dataSerialize({ type: this.id, isImmersive: this.isImmersive }));
    }
}
export class XR {
    static requestSession(sessionId, props) {
        if (!XR.currentSessionId) {
            XR.currentSessionId = sessionId;
            WindowManager.getWindow(sessionId, true).open(props);
        }
        return Promise.resolve();
    }
    static endSession() {
        if (XR.currentSessionId) {
            WindowManager.getWindow(XR.currentSessionId, true).close();
            XR.currentSessionId = null;
        }
        return Promise.resolve();
    }
}
//# sourceMappingURL=index.ios.js.map